import json
from tempfile import NamedTemporaryFile

import sensenova


def test_finetune_create():
    result = sensenova.FineTune.create(
        hyperparams= {
            "learning_rate": 0.0001,
            "lora_alpha": 32,
            "lora_dropout": 0.05,
            "lora_rank": 8,
            "lr_scheduler_type": "cosine",
            "max_steps": 10,
            "modules_to_save": "embed_tokens,lm_head",
            "save_steps": 10,
            "warmup_ratio": 0.03,
            "weight_decay": 0
        },
        model="nova-ptc-xs-v1",
        suffix="wz",
        training_file="12d1ba23-427c-42ad-af1a-d0b927b8e3b7"
    )
    print(result)


def test_finetune_list():
    resp = sensenova.FineTune.list()
    print(resp)

def test_finetune_cancel(id):
    resp = sensenova.FineTune.cancel(id)
    print(resp)


if __name__ == "__main__":
    id = 'ft-a3afd31f335447ac8157d8471789a11c'
    # test_finetune_list()
    # test_finetune_create()
    test_finetune_list()
    # res = sensenova.FineTune.retrieve(id=id)
    # print(res)
    # test_finetune_cancel(id=id)
    print(sensenova.FineTune.delete(sid=id))
    # test_finetune_list()



import json
import os
import sys
from typing import Any, Callable, NamedTuple, Optional


class Remediation(NamedTuple):
    name: str
    immediate_msg: Optional[str] = None
    necessary_msg: Optional[str] = None
    necessary_fn: Optional[Callable[[Any], Any]] = None
    optional_msg: Optional[str] = None
    optional_fn: Optional[Callable[[Any], Any]] = None
    error_msg: Optional[str] = None


def clean_data(metas):
    input_set = set()
    new_metas = []
    error_msg = None
    immediate_msg = None
    for meta in metas:
        if ("instruction" not in meta) or ("input" not in meta):
            error_msg = '[ERROR] Training item must have keys of instruction and input!!!'
        if "output" not in meta:
            error_msg = '[ERROR] Training item must have an output!!!'
            break
        if (meta['instruction'] + '###' + meta['input']) not in input_set:
            new_metas.append(meta)
            input_set.add(meta['instruction'] + '###' + meta['input'])
    if not error_msg:
        immediate_msg = f"Data length before clean: {len(metas)}\nData length after clean length: {len(new_metas)}\n"
    return new_metas, Remediation(name="num_examples", immediate_msg=immediate_msg, error_msg=error_msg)


def read_any_format(fname):
    remediation = None
    necessary_msg = None
    immediate_msg = None
    error_msg = None
    metas = None

    if os.path.isfile(fname):
        try:
            metas = json.load(open(fname, 'r'))
        except Exception:
            error_msg = f"Your file {fname} cannot be loaded in json format"
    else:
        error_msg = f"File {fname} does not exist."

    remediation = Remediation(
        name="read_any_format",
        necessary_msg=necessary_msg,
        immediate_msg=immediate_msg,
        error_msg=error_msg,
    )
    return metas, remediation


def apply_necessary_remediation(metas, remediation):
    if remediation.error_msg is not None:
        sys.stderr.write(
            f"\n\nERROR in {remediation.name}: {remediation.error_msg}\n\nAborting..."
        )
        sys.exit(1)
    if remediation.immediate_msg is not None:
        sys.stdout.write(remediation.immediate_msg)
    if remediation.necessary_fn is not None:
        metas = remediation.necessary_fn(metas)
    return metas


def get_processors():
    return [
        clean_data
    ]


def write_out_file(metas, fname):
    new_fname = f'cleaned_{os.path.basename(fname)}'
    json.dump(metas, open(new_fname, 'w', encoding='utf-8'), ensure_ascii=False)
    sys.stdout.write(f"The processed data is saved as {new_fname}\n")


def apply_processors(metas, fname, processors, write_out_file_func):
    for process in processors:
        metas, remediation = process(metas)
        metas = apply_necessary_remediation(metas, remediation)

    write_out_file_func(metas, fname)



import json
from copy import deepcopy
from typing import Optional

from sensenova import util, api_requestor


class SensenovaObject(dict):
    api_base_override = None

    def __init__(
        self,
        id=None,
        access_key_id=None,
        secret_access_key=None,
        api_base=None,
        **params,
    ):
        super(SensenovaObject, self).__init__()

        self._retrieve_params = params

        object.__setattr__(self, "access_key_id", access_key_id)
        object.__setattr__(self, "secret_access_key", secret_access_key)
        object.__setattr__(self, "api_base_override", api_base)
        if id:
            self["id"] = id

    def __setattr__(self, k, v):
        if k[0] == "_" or k in self.__dict__:
            return super(SensenovaObject, self).__setattr__(k, v)

        self[k] = v
        return None

    def __getattr__(self, k):
        if k[0] == '_':
            raise AttributeError(k)
        try:
            return self[k]
        except KeyError as err:
            raise AttributeError(*err.args)

    def __delattr__(self, k):
        if k[0] == "_" or k in self.__dict__:
            return super(SensenovaObject, self).__delattr__(k)
        else:
            del self[k]

    def __setitem__(self, k, v):
        if v == "":
            raise ValueError(
                "You cannot set %s to an empty string. "
                "We interpret empty strings as None in requests."
                "You may set %s.%s = None to delete the property" % (k, str(self), k)
            )
        super(SensenovaObject, self).__setitem__(k, v)

    def __delitem__(self, k):
        raise NotImplementedError("del is not supported")

    def __setstate__(self, state):
        self.update(state)

    def __reduce__(self):
        reduce_value = (
            type(self),
            (
                self.get("id", None),
                self.access_key_id, self.secret_access_key
            ),
            dict(self)
        )
        return reduce_value

    @classmethod
    def construct_from(
        cls,
        values,
        access_key_id: Optional[str] = None,
        secret_access_key: Optional[str] = None
    ):
        instance = cls(
            values.get("id"),
            access_key_id=access_key_id, secret_access_key=secret_access_key
        )
        instance.refresh_from(
            values,
            access_key_id=access_key_id, secret_access_key=secret_access_key,
        )
        return instance

    def refresh_from(
        self,
        values,
        access_key_id=None,
        secret_access_key=None
    ):
        self.access_key_id = access_key_id or getattr(values, "access_key_id", None)
        self.secret_access_key = secret_access_key or getattr(values, "secret_access_key", None)

        self.clear()
        for k, v in values.items():
            super(SensenovaObject, self).__setitem__(
                k, util.convert_to_sensenova_object(v, access_key_id, secret_access_key)
            )

        self._previous = values

    @classmethod
    def api_base(cls):
        return None

    def request(
        self,
        method,
        url,
        params=None,
        headers=None,
        stream=False,
        plain_old_data=False
    ):
        if params is None:
            params = self._retrieve_params

        requestor = api_requestor.APIRequestor(
            access_key_id=self.access_key_id,
            secret_access_key=self.secret_access_key,
            api_base=self.api_base_override or self.api_base(),
        )

        response, stream, access_key_id, secret_access_key = requestor.request(
            method,
            url,
            params=params,
            stream=stream,
            headers=headers
        )

        if stream:
            assert not isinstance(response, SensenovaObject)
            return (
                util.convert_to_sensenova_object(
                    line,
                    access_key_id,
                    secret_access_key,
                    plain_old_data=plain_old_data
                )
                for line in response
            )
        else:
            return util.convert_to_sensenova_object(
                response,
                access_key_id, secret_access_key,
                plain_old_data=plain_old_data
            )

    async def arequest(
        self,
        method,
        url,
        params=None,
        headers=None,
        stream=False,
        plain_old_data=False
    ):
        if params is None:
            params = self._retrieve_params

        requestor = api_requestor.APIRequestor(
            access_key_id=self.access_key_id,
            secret_access_key=self.secret_access_key,
            api_base=self.api_base_override or self.api_base(),
        )

        response, stream, access_key_id, secret_access_key = await requestor.arequest(
            method,
            url,
            params=params,
            stream=stream,
            headers=headers
        )

        if stream:
            assert not isinstance(response, SensenovaObject)
            return (
                util.convert_to_sensenova_object(
                    line,
                    access_key_id, secret_access_key,
                    plain_old_data=plain_old_data
                )
                for line in response
            )
        else:
            return util.convert_to_sensenova_object(
                response,
                access_key_id, secret_access_key,
                plain_old_data=plain_old_data
            )

    def __copy__(self):
        copied = SensenovaObject(
            self.get("id"),
            self.access_key_id, self.secret_access_key
        )

        copied._retrieve_params = self._retrieve_params

        for k, v in self.items():
            super(SensenovaObject, copied).__setitem__(k, v)

        return copied

    def __deepcopy__(self, memo):
        copied = self.__copy__()
        memo[id(self)] = copied

        for k, v in self.items():
            super(SensenovaObject, copied).__setitem__(k, deepcopy(v, memo))

        return copied

    def __repr__(self):
        ident_parts = [type(self).__name__]

        obj = self.get("object")
        if isinstance(obj, str):
            ident_parts.append(obj)

        if isinstance(self.get("id"), str):
            ident_parts.append("id=%s" % (self.get("id"),))

        unicode_repr = "<%s at %s> JSON: %s" % (
            " ".join(ident_parts),
            hex(id(self)),
            str(self),
        )

        return unicode_repr

    def __str__(self):
        obj = self.to_dict_recursive()
        return json.dumps(obj, sort_keys=True, indent=2)

    def to_dict(self):
        return dict(self)

    def to_dict_recursive(self):
        d = dict(self)
        for k, v in d.items():
            if isinstance(v, SensenovaObject):
                d[k] = v.to_dict_recursive()
            elif isinstance(v, list):
                d[k] = [
                    e.to_dict_recursive() if isinstance(e, SensenovaObject) else e
                    for e in v
                ]
        return d



from sensenova.api_resources.abstract.api_resource import APIResource
from sensenova.api_resources.abstract.listable_api_resource import ListableAPIResource
from sensenova.api_resources.abstract.createable_api_resource import CreatableAPIResource
from sensenova.api_resources.abstract.deletable_api_resource import DeletableAPIResource
from sensenova.api_resources.abstract.engine_api_resource import EngineAPIResource
from sensenova.api_resources.abstract.fileable_api_resource import FileableAPIResource
from sensenova.api_resources.abstract.downloadable_api_resource import DownloadableAPIResource
from sensenova.api_resources.abstract.uploadable_api_resource import UploadableAPIResource


import io
import json
from tempfile import NamedTemporaryFile
import sensenova


def test_knowledge_base_add_file(id, description="1Test client??"):
    result = sensenova.KnowledgeBase.add_file(id=id, description=description)
    print(result)
    return result["url"]


def test_knowledge_base_list():
    result = sensenova.KnowledgeBase.list()
    print(result)


def test_knowledge_base_retrieve(id="mrkd"):
    result = sensenova.KnowledgeBase.retrieve(id=id)
    print(result)


def test_knowledge_base_delete(id):
    result = sensenova.KnowledgeBase.delete(sid=id)
    print(result)


def test_knowledge_base_upload(url):
    payload = {
        "qa_lst": [
            {
                "std_q": "xxx",
                "simi_qs": ["xxx", "xxx"],
                "answer": "xxx",
            },
            {
                "std_q": "xxx",
                "simi_qs": ["xxx", "xxx"],
                "answer": "xxx",
            },
        ],
        "text_lst": [
            "xxx",
            "xxx"
        ]
    }

    result = sensenova.KnowledgeBase.upload_file(aoss_url=url, file=io.StringIO(json.dumps(payload, ensure_ascii=False)))
    print(result)


def test_knowledge_base_download(id="mrkd", file_id="1"):
    result = sensenova.KnowledgeBase.download(id=id, file_id=file_id)
    print(result.decode('utf-8'))


def test_knowledge_base_create(description="Test client"):
    result = sensenova.KnowledgeBase.create(description=description)
    print(result)


# test_knowledge_base_list()
if __name__ == "__main__":
    id = "sf88e224517394ba0aceb9917efefd123"
    file_id = "9cb27fb844d04ef9b16806843f63ca15"
    test_knowledge_base_list()
    # test_knowledge_base_delete(id=id)
    # test_knowledge_base_download(id=id)
    # test_knowledge_base_create()
    # url = test_knowledge_base_add_file(id=id, description="How are YOU!")
    # test_knowledge_base_upload(url)
    # test_knowledge_base_retrieve(id=id)
    # test_knowledge_base_delete(id=id)
    # test_knowledge_base_list()
    test_knowledge_base_download(id=id, file_id=file_id)



from typing import Optional

from sensenova.sensenova_object import SensenovaObject
from sensenova.util import merge_dicts


class ErrorObject(SensenovaObject):
    def refresh_from(
        self,
        values,
        access_key_id=None,
        secret_access_key=None,
        response_ms: Optional[int] = None,
    ):
        values = merge_dicts({"message": None, "type": None}, values)
        return super(ErrorObject, self).refresh_from(
            values=values,
            access_key_id=access_key_id,
            secret_access_key=access_key_id,
        )


from sensenova.api_resources.abstract import DeletableAPIResource, ListableAPIResource, CreatableAPIResource
from urllib.parse import quote_plus


class FineTune(ListableAPIResource, DeletableAPIResource, CreatableAPIResource):
    OBJECT_NAME = "llm.fine-tunes"

    @classmethod
    def _prepare_cancel(
        cls,
        id,
        access_key_id=None,
        secret_access_key=None,
        **params
    ):
        base = cls.class_url()
        extn = quote_plus(id)

        url = "%s/%s/cancel" % (base, extn)

        instance = cls(id, access_key_id, secret_access_key, **params)
        return instance, url

    @classmethod
    def cancel(cls, id, access_key_id=None, secret_access_key=None, **params):
        instance, url = cls._prepare_cancel(
            id,
            access_key_id,
            secret_access_key,
            **params
        )
        return instance.request("post", url)

    @classmethod
    async def acancel(cls, id, access_key_id=None, secret_access_key=None, **params):
        instance, url = cls._prepare_cancel(
            id,
            access_key_id, secret_access_key,
            **params
        )
        return await instance.arequest("post", url)



import io
import json
from tempfile import NamedTemporaryFile

import sensenova


def test_dataset_add_file(id, description="1Test client??"):
    result = sensenova.Dataset.add_file(id=id, description=description)
    print(result)
    return result["url"]


def test_dataset_list():
    result = sensenova.Dataset.list()
    print(result)


def test_dataset_retrieve(id="mrkd"):
    result = sensenova.Dataset.retrieve(id=id)
    print(result)


def test_dataset_delete(id):
    result = sensenova.Dataset.delete(sid=id)
    print(result)


def test_dataset_upload(url):
    payload = [
        {
            "instruction": "我们如何在日常生活中减少用水？",
            "input": "",
            "output": "1. 使用节水装置，如节水淋浴喷头和水龙头。 \n2. 使用水箱或水桶收集家庭废水，例如洗碗和洗浴。 \n3. 在社区中提高节水意识。 \n4. 检查水管和灌溉系统的漏水情况，并及时修复它们。 \n5. 洗澡时间缩短，使用低流量淋浴头节约用水。 \n6. 收集雨水，用于园艺或其他非饮用目的。 \n7. 刷牙或擦手时关掉水龙头。 \n8. 减少浇水草坪的时间。 \n9. 尽可能多地重复使用灰水（来自洗衣机、浴室水槽和淋浴的水）。 \n10. 只购买能源效率高的洗碗机和洗衣机。"
        },
        {
            "instruction": "编辑文章，使其更吸引读者。",
            "input": "自主机器人是计算机控制的机器，被编程执行特定任务而不需要任何人类输入。自主机器人在各个行业中被越来越广泛地应用，从制造业到医疗保健再到安全。",
            "output": "自主机器人是计算机控制的机器，被编程执行特定任务而不需要任何人类输入，从而实现了新的效率、精确度和可靠性水平。自主机器人在各个行业中被越来越广泛地应用，从制造业，它们可以使用精度和一致的质量组装复杂的组件，到医疗保健，可以协助进行医疗测试和处理，再到安全，可以监控大面积地区，保障人们和财产的安全。自主机器人还可以减少在危险或有害环境中的错误和增加安全，在工业流程的检查或维修期间等。由于其多样性，自主机器人将彻底改变我们工作方式的方式，使任务变得更加简单、快速，最终更加愉悦。"
        }
    ]


    result = sensenova.Dataset.upload_file(aoss_url=url, file=io.StringIO(json.dumps(payload, ensure_ascii=False)))
    print(result)


def test_dataset_download(id="mrkd", file_id="1"):
    result = sensenova.Dataset.download(id=id, file_id=file_id)
    print(result.decode('utf-8'))


def test_dataset_create(description="Test client"):
    result = sensenova.Dataset.create(description=description)
    print(result)


# test_dataset_list()
if __name__ == "__main__":
    id = "8ba02d7f-ece5-439c-976c-444d02d74bf8"
    file_id = "a7cd2a40-747b-430c-8344-3dd62f02dfe8"
    # test_dataset_list()
    # test_dataset_delete(id=id)
    # test_dataset_download(id=id)
    # test_dataset_create()
    # url = test_dataset_add_file(id=id, description="How are YOU!")
    # print(url)
    # url = 'https://aoss.cn-sh-01.sensecoreapi-oss.cn/nova-test-training-data/ca249841-727f-4917-a747-a58ca3000cfc?X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential=35C99457BF6D484C812AE6C7FCC2C459%2F20230629%2Fcn-north-1%2Fs3%2Faws4_request&X-Amz-Date=20230629T060440Z&X-Amz-Expires=600&X-Amz-SignedHeaders=host&X-Amz-Signature=ab953d05b7dfacf81ebaad59004c856a7a9d31aaabddf8e473596ae668359c0c'
    # test_dataset_upload(url)
    # test_dataset_list()
    # test_dataset_retrieve(id=id)
    test_dataset_download(id=id, file_id=file_id)



from sensenova import api_requestor, util
from sensenova.api_resources.abstract.api_resource import APIResource


class UploadableAPIResource(APIResource):
    plain_old_data = False

    @classmethod
    def __prepare_upload_file(
        cls,
        aoss_url,
        file,
        access_key_id=None,
        secret_access_key=None,
    ):
        requestor = api_requestor.APIRequestor(
            access_key_id, secret_access_key,
            api_base=aoss_url[:12],
        )
        json_data = file.read()
        if hasattr(json_data, 'encode'):
            json_data = json_data.encode()
        return requestor, aoss_url[12:], json_data

    @classmethod
    def upload_file(
        cls,
        aoss_url,
        file,
        access_key_id=None,
        secret_access_key=None,
        **params
    ):
        requestor, url, content = cls.__prepare_upload_file(
            aoss_url, file, access_key_id, secret_access_key
        )

        response, _, access_key_id, secret_access_key = requestor.request(
            "put", url, params=content
        )
        return util.convert_to_sensenova_object(
            response,
            access_key_id, secret_access_key,
            plain_old_data=cls.plain_old_data
        )

    @classmethod
    async def aupload_file(
        cls,
        aoss_url,
        file,
        access_key_id=None,
        secret_access_key=None,
        **params
    ):
        requestor, url, content = cls.__prepare_upload_file(
            aoss_url, file, access_key_id, secret_access_key
        )

        response, _, access_key_id, secret_access_key = await requestor.arequest(
            "put", url, params=content
        )
        return util.convert_to_sensenova_object(
            response,
            access_key_id, secret_access_key,
            plain_old_data=cls.plain_old_data
        )



import json
import sensenova


def test_serving_create(model_id):
    result = sensenova.Serving.create(
        model=model_id,
        config={
            "run_time": 60
        }
    )
    print(result)


def test_serving_list():
    resp = sensenova.Serving.list()
    print(resp)


def test_serving_cancel(id):
    resp = sensenova.Serving.cancel(id)
    print(resp)


def test_serving_retrieve(id):
    resp = sensenova.Serving.retrieve(id)
    print(resp)


def test_serving_relaunch(id):
    resp = sensenova.Serving.relaunch(id)
    print(resp)


if __name__ == "__main__":
    id = "a0f0caee-1cfb-4a5c-b2ef-d041d80476cb"
    model_id = "llama-7b-test:wzh"
    # test_finetune_list()
    # test_serving_list()
    # test_serving_list()

    # test_serving_create(model_id)
    test_serving_cancel(id)
    # print(sensenova.Serving.delete(sid=id))
    test_serving_relaunch(id)
    # test_serving_retrieve(id)
    # test_serving_cancel()
    # test_serving_list()



from typing import Optional
from urllib.parse import quote_plus

from sensenova import api_requestor, util
from sensenova.api_resources.abstract import APIResource
from sensenova.sensenova_response import SensenovaResponse


class EngineAPIResource(APIResource):
    plain_old_data = False

    def __init__(self, engine: Optional[str] = None, **kwargs):
        super().__init__(engine=engine, **kwargs)

    @classmethod
    def class_url(
        cls,
        engine: Optional[str] = None
    ):
        base = cls.OBJECT_NAME.replace(".", "/")
        if engine is None:
            return "/%s" % base
        extn = quote_plus(engine)
        return "/engines/%s/%s" % (extn, base)

    @classmethod
    def __prepare_create_request(
        cls,
        access_key_id=None,
        secret_access_key=None,
        api_base=None,
        **params,
    ):
        stream = params.get("stream", False)
        headers = params.pop("headers", None)
        requestor = api_requestor.APIRequestor(
            access_key_id,
            secret_access_key,
            api_base=api_base
        )
        url = cls.class_url()
        return (
            stream,
            headers,
            requestor,
            url,
            params,
        )

    @classmethod
    def create(
        cls,
        access_key_id=None, secret_access_key=None,
        api_base=None,
        **params
    ):
        (
            stream,
            headers,
            requestor,
            url,
            params,
        ) = cls.__prepare_create_request(
            access_key_id, secret_access_key, api_base, **params
        )

        response, _, access_key_id, secret_access_key = requestor.request(
            "post",
            url,
            params=params,
            headers=headers,
            stream=stream,
        )

        if stream:
            assert not isinstance(response, SensenovaResponse)
            return (
                util.convert_to_sensenova_object(
                    line,
                    access_key_id, secret_access_key,
                    plain_old_data=cls.plain_old_data
                )
                for line in response
            )
        else:
            obj = util.convert_to_sensenova_object(
                response,
                access_key_id, secret_access_key,
                plain_old_data=cls.plain_old_data
            )

        return obj

    @classmethod
    async def acreate(
        cls,
        access_key_id=None, secret_access_key=None,
        api_base=None,
        **params
    ):
        (
            stream,
            headers,
            requestor,
            url,
            params,
        ) = cls.__prepare_create_request(
            access_key_id, secret_access_key, api_base, **params
        )

        response, _, access_key_id, secret_access_key = await requestor.arequest(
            "post",
            url,
            params=params,
            headers=headers,
            stream=stream,
        )

        if stream:
            assert not isinstance(response, SensenovaResponse)
            return (
                util.convert_to_sensenova_object(
                    line,
                    access_key_id, secret_access_key,
                    plain_old_data=cls.plain_old_data
                )
                async for line in response
            )
        else:
            obj = util.convert_to_sensenova_object(
                response,
                access_key_id, secret_access_key,
                plain_old_data=cls.plain_old_data
            )

        return obj



import sys

from sensenova.util import finetune_completions_option_params

import sensenova

stream = True
resp = sensenova.FinetuneCompletion.create(
    model_id="SC-PTC-S-V1-20230601-test_unlimit-lhk1-2023-06-13-06-37-09",
    text="想一个寓言故事",
    stream=stream
)

if not stream:
    resp = [resp]

for part in resp:
    content = part['answer']
    if content:
        sys.stdout.write(content)
    sys.stdout.flush()


from typing import Awaitable
from urllib.parse import quote_plus

from sensenova.api_resources.abstract.api_resource import APIResource


class DeletableAPIResource(APIResource):
    @classmethod
    def __prepare_delete(cls, sid):
        if isinstance(cls, APIResource):
            raise ValueError(".delete may only be called as a class method now.")

        base = cls.class_url()
        extn = quote_plus(sid)

        url = "%s/%s" % (base, extn)
        return url

    @classmethod
    def delete(cls, sid, **params):
        url = cls.__prepare_delete(sid)

        return cls._static_request(
            "delete", url, **params
        )

    @classmethod
    def adelete(cls, sid, **params) -> Awaitable:
        url = cls.__prepare_delete(sid)

        return cls._astatic_request(
            "delete", url, **params
        )



import sensenova


class SensenovaError(Exception):
    def __init__(
        self,
        message=None,
        http_body=None,
        http_status=None,
        json_body=None,
        headers=None,
        code=None
    ):
        super(SensenovaError, self).__init__(message)

        if http_body and hasattr(http_body, "decode"):
            try:
                http_body = http_body.decode("utf-8")
            except BaseException:
                http_body = (
                    "<Could not decode body as utf-8.>"
                )

        self._message = message
        self.http_body = http_body
        self.http_status = http_status
        self.json_body = json_body
        self.headers = headers or {}
        self.code = code
        self.error = self.construct_error_object()

    def __str__(self):
        msg = self._message or "<empty message>"
        return msg + '\n'

    def construct_error_object(self):
        if (
            self.json_body is None
            or not isinstance(self.json_body, dict)
            or "error" not in self.json_body
            or not isinstance(self.json_body["error"], dict)
        ):
            return None

        return sensenova.api_resources.error_object.ErrorObject.construct_from(
            self.json_body["error"]
        )


class APIConnectionError(SensenovaError):
    def __init__(
        self,
        message,
        http_body=None,
        http_status=None,
        json_body=None,
        headers=None,
        code=None,
        should_retry=False,
    ):
        super(APIConnectionError, self).__init__(
            message, http_body, http_status, json_body, headers, code
        )
        self.should_retry = should_retry


class InvalidRequestError(SensenovaError):
    def __init__(
        self,
        message,
        param=None,
        code=None,
        http_body=None,
        http_status=None,
        json_body=None,
        headers=None,
    ):
        super(InvalidRequestError, self).__init__(
            message, http_body, http_status, json_body, headers, code
        )
        self.param = param

    def __repr__(self):
        return "%s(message=%r, param=%r, code=%r, http_status=%r)" % (
            self.__class__.__name__,
            self._message,
            self.param,
            self.code,
            self.http_status,
        )

    def __reduce__(self):
        return type(self), (
            self._message,
            self.param,
            self.code,
            self.http_body,
            self.http_status,
            self.json_body,
            self.headers
        )


class ServiceUnavailableError(SensenovaError):
    pass


class APIError(SensenovaError):
    pass


class AuthenticationError(SensenovaError):
    pass


class RateLimitError(SensenovaError):
    pass


class PermissionError(SensenovaError):
    pass


class TryAgain(SensenovaError):
    pass



from sensenova import api_requestor, util
from sensenova.api_resources.abstract.api_resource import APIResource


class ListableAPIResource(APIResource):
    @classmethod
    def auto_paging_iter(cls, *args, **params):
        return cls.list(*args, **params).auto_paging_iter()

    @classmethod
    def __prepare_list_requestor(cls, access_key_id=None, secret_access_key=None, api_base=None):
        requestor = api_requestor.APIRequestor(
            access_key_id,secret_access_key,
            api_base=api_base or cls.api_base()
        )

        url = cls.class_url()
        return requestor, url

    @classmethod
    def list(
        cls,
        access_key_id=None, secret_access_key=None,
        api_base=None,
        **params
    ):
        requestor, url = cls.__prepare_list_requestor(
            access_key_id, secret_access_key,
            api_base
        )

        response, _, access_key_id, secret_access_key = requestor.request(
            "get", url, params
        )

        sensenova_object = util.convert_to_sensenova_object(
            response, access_key_id, secret_access_key
        )

        # sensenova_object._retrieve_params = params
        return sensenova_object


    @classmethod
    async def alist(
        cls,
        access_key_id=None, secret_access_key=None,
        api_base=None,
        **params
    ):
        requestor, url = cls.__prepare_list_requestor(
            access_key_id, secret_access_key,
            api_base
        )

        response, _, access_key_id, secret_access_key = await requestor.arequest(
            "get", url, params
        )

        sensenova_object = util.convert_to_sensenova_object(
            response, access_key_id, secret_access_key
        )

        # sensenova_object._retrieve_params = params
        return sensenova_object


import logging
import os
import re
import sys
import time
import jwt
import sensenova.sensenova_object
from sensenova import sensenova_response
import sensenova

SENSENOVA_LOG = os.environ.get("SENSENOVA_LOG")

logger = logging.getLogger("sensenova")


def ak_sk_to_header(access_key_id, secret_access_key):
    headers = {
        "alg": "HS256",
        "typ": "JWT"
    }
    payload = {
        "iss": access_key_id,
        "exp": int(time.time()) + 1800,  # 有效时间
        "nbf": int(time.time()) - 5  # 签发时间
    }
    token = jwt.encode(payload, secret_access_key, headers=headers)
    return {
        "Authorization": f"Bearer {token}"
    }


def finetune_completions_option_params():
    return [
        ["temperature", "Temperature sampling parameter, the value is (0,2]. Values greater than 1 tend to generate "
                        "more diverse replies, and values less than 1 tend to generate more stable replies. Default "
                        "0.8", True],
        ["top_p", "Kernel sampling parameter, the value is (0,1]."
                  " When decoding and generating tokens, sampling is performed in the minimum token set whose probability"
                  " sum is greater than or equal to top_p. "
                  "Default 0.7", True],
        ["max_new_tokens", "The maximum number of tokens generated. Note: input_tokens + max_new_tokens <= 1512", True],
        ["repetition_penalty", "Repeat penalty factor, 1 means no penalty, "
                               "greater than 1 tends to generate non-repeat token, "
                               "less than 1 tends to generate repeat token."
                               " Default 1", True],
    ]


def fine_tunes_hyperparams():
    # name, help, if eval
    return [
        ["max_steps", "The maximum number of training iterations.", True],
        ["lr_scheduler_type", "Learning Rate Adjustment Strategy.", False],
        ["learning_rate", "Learning rate.", True],
        ["warmup_ratio", "Warmup iteration ratio.", True],
        ["weight_decay", "Regularized weight decay.", True],
        ["save_steps", "Weight saving iteration interval.", True],
        ["modules_to_save", "Modules that need to be trained and saved.", False],
        ["lora_rank", "The rank of the lora layer.", True],
        ["lora_dropout", "The dropout probability of the lora layer.", True],
        ["lora_alpha", "The regularization strength of the lora layer.", True]
    ]


def logfmt(props):
    def fmt(key, val):
        if hasattr(val, 'decode'):
            val = val.decode("utf-8")

        if not isinstance(val, str):
            val = str(val)

        if re.search(r"\s", val):
            val = repr(val)

        if re.search(r"\s", key):
            key = repr(key)
        return "{key}={val}.".format(key=key, val=val)

    return " ".join([fmt(key, val) for key, val in sorted(props.items())])


def _console_log_level():
    if sensenova.log in ["debug", "info"]:
        return sensenova.log
    elif SENSENOVA_LOG in ["debug", "info"]:
        return SENSENOVA_LOG
    else:
        return None


def log_debug(message, **params):
    msg = logfmt(dict(message=message, **params))
    if _console_log_level() == "debug":
        print(msg, file=sys.stderr)
    logger.debug(msg)


def default_access_key_id() -> str:
    if sensenova.access_key_id is not None:
        return sensenova.access_key_id
    else:
        raise sensenova.error.AuthenticationError(
            "No access key id provided. You can set your access key id in code using 'sensenova.access_key_id = <ACCESS-KEY-ID>', or you can set the environment variable "
            "SENSENOVA_ACCESS_KEY_ID=<ACCESS-KEY-ID>)."
        )


def default_secret_access_key() -> str:
    if sensenova.secret_access_key is not None:
        return sensenova.secret_access_key
    else:
        raise sensenova.error.AuthenticationError(
            "No secret access key id provided. You can set your secret access key in code using 'sensenova.secret_access_key = <SECRET-ACCESS-KEY>', or you can set the environment variable "
            "SENSENOVA_SECRET_ACCESS_KEY=<SECRET-ACCESS-KEY>)."
        )


def get_object_classes():
    from sensenova.object_classes import OBJECT_CLASSES

    return OBJECT_CLASSES


def convert_to_sensenova_object(resp, access_key_id=None, secret_access_key=None, plain_old_data=False):
    if isinstance(resp, sensenova_response.SensenovaResponse):
        resp = resp.data

    if plain_old_data:
        return resp
    elif isinstance(resp, list):
        return [
            convert_to_sensenova_object(
                i, access_key_id, secret_access_key
            )
            for i in resp
        ]
    elif isinstance(resp, dict) and not isinstance(
        resp, sensenova.sensenova_object.SensenovaObject
    ):
        resp = resp.copy()
        klass_name = resp.get("object")
        if isinstance(klass_name, str):
            klass = get_object_classes().get(
                klass_name, sensenova.sensenova_object.SensenovaObject
            )
        else:
            klass = sensenova.sensenova_object.SensenovaObject

        return klass.construct_from(
            resp,
            access_key_id=access_key_id, secret_access_key=secret_access_key
        )
    else:
        return resp


def log_debug(message, **params):
    msg = logfmt(dict(message=message, **params))
    if _console_log_level() == "debug":
        print(msg, file=sys.stderr)
    logger.debug(msg)


def log_info(message, **params):
    msg = logfmt(dict(message=message, **params))
    if _console_log_level() in ["debug", "info"]:
        print(msg, file=sys.stderr)
    logger.info(msg)


def merge_dicts(x, y):
    z = x.copy()
    z.update(y)
    return z



from sensenova import error
from sensenova.api_resources.abstract import EngineAPIResource

class FinetuneCompletion(EngineAPIResource):
    OBJECT_NAME = "nlp.finetune.completions"

    @classmethod
    def create(cls, *args, **kwargs):
        text = kwargs.get("text", None)
        model_id = kwargs.get("model_id", None)

        if not text:
            raise error.InvalidRequestError(
                "The parameter text is required"
            )

        if not model_id:
            raise error.InvalidRequestError(
                "The parameter model_id is required"
            )

        return super().create(*args, **kwargs)


    @classmethod
    async def acreate(cls, *args, **kwargs):
        text = kwargs.get("text", None)
        model_id = kwargs.get("model_id", None)

        if not text:
            raise error.InvalidRequestError(
                "The parameter text is required"
            )

        if not model_id:
            raise error.InvalidRequestError(
                "The parameter model_id is required"
            )

        return await super().acreate(*args, **kwargs)


from urllib.parse import quote_plus
import json
from typing import cast
from urllib.parse import quote_plus

import sensenova

from sensenova import api_requestor, util
from sensenova.api_resources.abstract.api_resource import APIResource


class DownloadableAPIResource(APIResource):
    plain_old_data = False

    @classmethod
    def __prepare_file_download(
        cls,
        id,
        file_id,
        access_key_id=None,
        secret_access_key=None,
        api_base=None
    ):
        requestor = api_requestor.APIRequestor(
            access_key_id, secret_access_key,
            api_base=api_base or sensenova.api_base,
        )
        url = f"{cls.class_url()}/{quote_plus(id)}/files/{file_id}"

        return requestor, url

    @classmethod
    def download(
        cls,
        id,
        file_id,
        access_key_id=None, secret_access_key=None,
        api_base=None
    ):
        requestor, url = cls.__prepare_file_download(
            id, file_id, access_key_id, secret_access_key, api_base
        )

        result = requestor.request_raw("get", url)
        if not 200 <= result.status_code < 300:
            raise requestor.handle_error_response(
                result.content,
                result.status_code,
                json.loads(cast(bytes, result.content)),
                result.headers,
                stream_error=False,
            )
        return result.content

    # @classmethod
    # async def adownload(
    #     cls,
    #     id,
    #     file_id,
    #     access_key_id=None, secret_access_key=None,
    #     api_base=None
    # ):
    #     requestor, url = cls.__prepare_file_download(
    #         id, file_id, access_key_id, secret_access_key, api_base
    #     )
    #     async with api_requestor.aiohttp_session() as session:
    #         result = await requestor.arequest_raw("get", url, session)
    #         if not 200 <= result.status < 300:
    #             raise requestor.handle_error_response(
    #                 result.content,
    #                 result.status,
    #                 json.loads(cast(bytes, result.content)),
    #                 result.headers,
    #                 stream_error=False,
    #             )
    #         return result.content



import os
import sys

# current_directory = os.path.dirname(os.path.abspath(__file__))
# sys.path.append(os.path.join(current_directory, '../..'))
# print()

import sensenova

id = "sc-ptc-xl-v1-20230609"
print(sensenova.Model.list())
# print(sensenova.Model.retrieve(id=id))
# print(sensenova.Model.delete(sid=id))
# print(sensenova.Model.list())



from urllib.parse import quote_plus

from sensenova import api_requestor, util
from sensenova.api_resources.abstract.api_resource import APIResource


class FileableAPIResource(APIResource):
    plain_old_data = False

    @classmethod
    def __prepare_file_requestor(
        cls,
        id: str,
        access_key_id=None, secret_access_key=None,
        api_base=None
    ):
        requestor = api_requestor.APIRequestor(
            access_key_id,
            secret_access_key,
            api_base=api_base,
        )

        url = f"{cls.class_url()}/{quote_plus(id)}/files"
        return requestor, url

    @classmethod
    def add_file(
        cls,
        id: str,
        access_key_id=None, secret_access_key=None,
        api_base=None,
        **params
    ):
        requestor, url = cls.__prepare_file_requestor(
            id,
            access_key_id, secret_access_key,
            api_base
        )

        response, _, access_key_id, secret_access_key = requestor.request(
            "post", url, params
        )

        return util.convert_to_sensenova_object(
            response,
            access_key_id, secret_access_key,
            plain_old_data=cls.plain_old_data
        )

    @classmethod
    async def aadd_file(
        cls,
        id,
        access_key_id=None, secret_access_key=None,
        api_base=None,
        **params
    ):
        requestor, url = cls.__prepare_file_requestor(
            id,
            access_key_id, secret_access_key,
            api_base
        )

        response, _, access_key_id, secret_access_key = await requestor.arequest(
            "post", url, params
        )

        return util.convert_to_sensenova_object(
            response,
            access_key_id, secret_access_key,
            plain_old_data=cls.plain_old_data
        )



import sys
from argparse import ArgumentParser

import sensenova.cli
from sensenova.util import fine_tunes_hyperparams, finetune_completions_option_params
from sensenova.validators import (
    apply_necessary_remediation,
    apply_processors,
    get_processors,
    read_any_format,
    write_out_file,
)
from sensenova.upload_progress import BufferReader


class KnowledgeBase:
    @classmethod
    def create(cls, args):
        resp = sensenova.KnowledgeBase.create(
            description=args.description
        )
        print(resp)

    @classmethod
    def add_file(cls, args):
        add_resp = sensenova.KnowledgeBase.add_file(id=args.id, description=args.description)
        with open(args.file, "rb") as file_reader:
            buffer_reader = BufferReader(file_reader.read(), desc="Upload progress")
        resp = sensenova.KnowledgeBase.upload_file(
            aoss_url=add_resp["url"],
            file=buffer_reader
        )
        print(resp)

    @classmethod
    def get(cls, args):
        resp = sensenova.KnowledgeBase.retrieve(id=args.id)
        print(resp)

    @classmethod
    def delete(cls, args):
        resp = sensenova.KnowledgeBase.delete(args.id)
        print(resp)

    @classmethod
    def list(cls, args):
        resp = sensenova.KnowledgeBase.list()
        print(resp)

    @classmethod
    def download(cls, args):
        resp = sensenova.KnowledgeBase.download(id=args.id, file_id=args.file_id)
        print(resp.decode("utf-8"))


class Dataset:
    @classmethod
    def create(cls, args):
        resp = sensenova.Dataset.create(
            description=args.description
        )
        print(resp)

    @classmethod
    def add_file(cls, args):
        add_resp = sensenova.Dataset.add_file(id=args.id, description=args.description)
        with open(args.file, "rb") as file_reader:
            buffer_reader = BufferReader(file_reader.read(), desc="Upload progress")
        resp = sensenova.Dataset.upload_file(
            aoss_url=add_resp["url"],
            file=buffer_reader
        )
        print(resp)

    @classmethod
    def get(cls, args):
        resp = sensenova.Dataset.retrieve(id=args.id)
        print(resp)

    @classmethod
    def delete(cls, args):
        resp = sensenova.Dataset.delete(args.id)
        print(resp)

    @classmethod
    def list(cls, args):
        resp = sensenova.Dataset.list()
        print(resp)

    @classmethod
    def download(cls, args):
        resp = sensenova.Dataset.download(id=args.id, file_id=args.file_id)
        print(resp.decode("utf-8"))


class Model:
    @classmethod
    def get(cls, args):
        resp = sensenova.Model.retrieve(id=args.id)
        print(resp)

    @classmethod
    def delete(cls, args):
        resp = sensenova.Model.delete(args.id)
        print(resp)

    @classmethod
    def list(cls, args):
        resp = sensenova.Model.list()
        print(resp)


class Serving:
    @classmethod
    def list(cls, args):
        resp = sensenova.Serving.list()
        print(resp)

    @classmethod
    def create(cls, args):
        create_args = {
            "model": args.model,
            "config": {
                "run_time": args.run_time
            }
        }
        resp = sensenova.Serving.create(**create_args)
        print(resp)

    @classmethod
    def get(cls, args):
        resp = sensenova.Serving.retrieve(id=args.id)
        print(resp)

    @classmethod
    def delete(cls, args):
        resp = sensenova.Serving.delete(args.id)
        print(resp)

    @classmethod
    def cancel(cls, args):
        resp = sensenova.Serving.cancel(id=args.id)
        print(resp)

    @classmethod
    def relaunch(cls, args):
        resp = sensenova.Serving.relaunch(id=args.id)
        print(resp)


class ChatCompletion:
    @classmethod
    def create(cls, args):
        messages = [
            {"role": role, "content": content} for role, content in args.message
        ]
        created_args = {
            "messages": messages
        }
        for param in (
            "model",
            "know_ids",
            "temperature",
            "top_p",
            "max_new_tokens",
            "repetition_penalty",
            "stream",
            "user",
        ):
            attr = getattr(args, param)
            if attr is not None:
                created_args[param] = attr

        resp = sensenova.ChatCompletion.create(
            **created_args
        )

        if not args.stream:
            resp = [resp]

        for part in resp:
            choices = part['data']["choices"]
            for c_idx, c in enumerate(choices):
                if len(choices) > 1:
                    sys.stdout.write("===== Chat Completion {} =====\n".format(c_idx))
                if args.stream:
                    delta = c.get("delta")
                    if delta:
                        sys.stdout.write(delta)
                else:
                    sys.stdout.write(c["message"])
                    if len(choices) > 1:  # not in streams
                        sys.stdout.write("\n")
                sys.stdout.flush()
        print()


class FinetuneCompletion:
    @classmethod
    def create(cls, args):
        created_args = {
            "text": args.text,
            "model_id": args.model_id,
            "stream": args.stream
        }

        for param in finetune_completions_option_params():
            attr = getattr(args, param[0])

            if attr is not None:
                if param[2]:
                    attr = eval(attr)
                created_args[param[0]] = attr

        resp = sensenova.FinetuneCompletion.create(**created_args)

        if not args.stream:
            resp = [resp]

        for part in resp:
            content = part['answer']
            if content:
                sys.stdout.write(content)
            sys.stdout.flush()


class FineTune:
    @classmethod
    def list(cls, args):
        resp = sensenova.FineTune.list()
        print(resp)

    @classmethod
    def create(cls, args):
        create_args = {
            "model": args.model,
            "training_file": args.training_file,
            "suffix": args.suffix
        }
        hyperparams = {}
        params = fine_tunes_hyperparams()
        for param in params:
            attr = getattr(args, param[0])
            if attr is not None:
                if param[2]:
                    attr = eval(attr)
                hyperparams[param[0]] = attr
        create_args["hyperparams"] = hyperparams
        resp = sensenova.FineTune.create(**create_args)
        print(resp)

    @classmethod
    def get(cls, args):
        resp = sensenova.FineTune.retrieve(id=args.id)
        print(resp)

    @classmethod
    def delete(cls, args):
        resp = sensenova.FineTune.delete(args.id)
        print(resp)

    @classmethod
    def cancel(cls, args):
        resp = sensenova.FineTune.cancel(id=args.id)
        print(resp)

    @classmethod
    def prepare_data(cls, args):
        sys.stdout.write("Analyzing...\n")
        fname = args.file
        metas, remediation = read_any_format(fname)
        apply_necessary_remediation(None, remediation)
        processors = get_processors()

        apply_processors(
            metas,
            fname,
            processors,
            write_out_file_func=write_out_file
        )


def tools_register(parser):
    subparsers = parser.add_subparsers(
        title="Tools", help="Convenience client side tools"
    )

    def help(args):
        parser.print_help()

    parser.set_defaults(func=help)

    sub = subparsers.add_parser("fine_tunes.prepare_data")
    sub.add_argument(
        "-f",
        "--file",
        required=True,
        help="Json file contains a data list, each data contains three fields instruction, input, output"
             "This should be the local file path.",
    )
    sub.set_defaults(func=FineTune.prepare_data)


def api_register(parser: ArgumentParser):
    subparsers = parser.add_subparsers(help="All API subcommands")

    def help(args):
        parser.print_help()

    parser.set_defaults(func=help)

    # knowledge-bases
    sub = subparsers.add_parser("knowledge-bases.create")

    sub.add_argument(
        "-d",
        "--description",
        required=False,
        help="Dataset description"
    )
    sub.set_defaults(func=KnowledgeBase.create)

    sub = subparsers.add_parser("knowledge-bases.add_file")
    sub.add_argument("-i", "--id", required=True, help="The knowledge bases ID")
    sub.add_argument(
        "-d",
        "--description",
        required=False,
        help="Description of the file to be added in the knowledge bases"
    )
    sub.add_argument(
        "-f",
        "--file",
        help="Path of the json file to upload"
    )
    sub.set_defaults(func=KnowledgeBase.add_file)

    sub = subparsers.add_parser("knowledge-bases.get")
    sub.add_argument("-i", "--id", required=True, help="The knowledge bases ID")
    sub.set_defaults(func=KnowledgeBase.get)

    sub = subparsers.add_parser("knowledge-bases.delete")
    sub.add_argument("-i", "--id", required=True, help="The knowledge bases ID")
    sub.set_defaults(func=KnowledgeBase.delete)

    sub = subparsers.add_parser("knowledge-bases.list")
    sub.set_defaults(func=KnowledgeBase.list)

    sub = subparsers.add_parser("knowledge-bases.download")
    sub.add_argument("-i", "--id", required=True, help="The knowledge bases ID")
    sub.add_argument("-f", "--file_id", required=True, help="The knowledge bases file ID")
    sub.set_defaults(func=KnowledgeBase.download)
    # datasets
    sub = subparsers.add_parser("datasets.create")

    sub.add_argument(
        "-d",
        "--description",
        required=False,
        help="Dataset description"
    )
    sub.set_defaults(func=Dataset.create)

    sub = subparsers.add_parser("datasets.add_file")
    sub.add_argument("-i", "--id", required=True, help="The datasets ID")
    sub.add_argument(
        "-d",
        "--description",
        required=False,
        help="Description of the file to be added in the Dataset"
    )
    sub.add_argument(
        "-f",
        "--file",
        help="Path of the json file to upload"
    )
    sub.set_defaults(func=Dataset.add_file)

    sub = subparsers.add_parser("datasets.get")
    sub.add_argument("-i", "--id", required=True, help="The datasets ID")
    sub.set_defaults(func=Dataset.get)

    sub = subparsers.add_parser("datasets.delete")
    sub.add_argument("-i", "--id", required=True, help="The datasets ID")
    sub.set_defaults(func=Dataset.delete)

    sub = subparsers.add_parser("datasets.list")
    sub.set_defaults(func=Dataset.list)

    sub = subparsers.add_parser("datasets.download")
    sub.add_argument("-i", "--id", required=True, help="The datasets ID")
    sub.add_argument("-f", "--file_id", required=True, help="The datasets file ID")
    sub.set_defaults(func=Dataset.download)

    # models
    sub = subparsers.add_parser("models.get")
    sub.add_argument("-i", "--id", required=True, help="The models ID")
    sub.set_defaults(func=Model.get)

    sub = subparsers.add_parser("models.delete")
    sub.add_argument("-i", "--id", required=True, help="The models ID")
    sub.set_defaults(func=Model.delete)

    sub = subparsers.add_parser("models.list")
    sub.set_defaults(func=Model.list)

    # fine_tunes
    sub = subparsers.add_parser("fine_tunes.list")
    sub.set_defaults(func=FineTune.list)

    sub = subparsers.add_parser("fine_tunes.create")
    sub.add_argument(
        "-t",
        "--training_file",
        required=True,
        help="JSON file containing prompt-completion examples for training. This can "
             "be the ID of a file uploaded through the Sensenova API,"
    )

    sub.add_argument(
        "-m",
        "--model",
        required=True,
        help="The model to start fine-tuning from",
    )
    sub.add_argument(
        "--suffix",
        required=True,
        help="This argument can be used to customize the generated fine-tuned model name."
             "The generated name will match the form `{base_model}-{user}:{suffix}-{timestamp}`. "
    )
    for param in fine_tunes_hyperparams():
        sub.add_argument(
            f"--{param[0]}",
            help=param[1]
        )
    sub.set_defaults(func=FineTune.create)

    sub = subparsers.add_parser("fine_tunes.get")
    sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune")
    sub.set_defaults(func=FineTune.get)

    sub = subparsers.add_parser("fine_tunes.cancel")
    sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune")
    sub.set_defaults(func=FineTune.cancel)

    sub = subparsers.add_parser("fine_tunes.delete")
    sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune")
    sub.set_defaults(func=FineTune.delete)

    # servings
    sub = subparsers.add_parser("servings.list")
    sub.set_defaults(func=Serving.list)

    sub = subparsers.add_parser("servings.create")
    sub.add_argument(
        "-m",
        "--model",
        required=True,
        help="The id of the finetune model"
    )
    sub.add_argument(
        "--run_time",
        required=True,
        type=int,
        help="Specifies the running time, in minutes"
    )
    sub.set_defaults(func=Serving.create)

    sub = subparsers.add_parser("servings.get")
    sub.add_argument("-i", "--id", required=True, help="The id of the serving")
    sub.set_defaults(func=Serving.get)

    sub = subparsers.add_parser("servings.delete")
    sub.add_argument("-i", "--id", required=True, help="The id of the serving")
    sub.set_defaults(func=Serving.delete)

    sub = subparsers.add_parser("servings.cancel")
    sub.add_argument("-i", "--id", required=True, help="The id of the serving")
    sub.set_defaults(func=Serving.cancel)

    sub = subparsers.add_parser("servings.relaunch")
    sub.add_argument("-i", "--id", required=True, help="The id of the serving")
    sub.set_defaults(func=Serving.relaunch)

    # Chat Completions
    sub = subparsers.add_parser("chat_completions.create")

    req = sub.add_argument_group("required arguments")
    opt = sub.add_argument_group("optional arguments")

    req.add_argument(
        "-g",
        "--message",
        action="append",
        nargs=2,
        metavar=("ROLE", "CONTENT"),
        help="A message in `{role} {content}` format. Use this argument multiple times to add multiple messages.",
        required=True,
    )
    req.add_argument(
        "-m",
        "--model",
        help="Model id. Default sensechat-001."
    )
    opt.add_argument(
        "--know_ids",
        nargs='+',
        help="List of knowledge bases id. For example, --know_ids xxx1 xxx2 xxx3."
    )
    opt.add_argument(
        "--temperature",
        type=float,
        help="Temperature sampling parameter,"
             "the value is (0,2]."
             "Values greater than 1 tend to generate more diverse replies, and values less than 1 tend to generate more "
             "stable replies. "
             "Default 0.8"
    )
    opt.add_argument(
        "--top_p",
        type=float,
        help="Kernel sampling parameter, the value is (0,1]."
             " When decoding and generating tokens, sampling is performed in the minimum token set whose probability"
             " sum is greater than or equal to top_p. "
             "Default 0.7"
    )
    opt.add_argument(
        "--max_new_tokens",
        type=int,
        help="The maximum number of tokens generated."
             " Default 2048."
    )
    opt.add_argument(
        "--repetition_penalty",
        type=float,
        help="Repeat penalty factor, 1 means no penalty, "
             "greater than 1 tends to generate non-repeat token, "
             "less than 1 tends to generate repeat token."
             " Default 1"
    )
    opt.add_argument(
        "--stream", help="Stream messages as they're ready.", action="store_true"
    )
    opt.add_argument(
        "--user",
        help="User id."
    )
    sub.set_defaults(func=ChatCompletion.create)

    # finetune_completion
    # sub = subparsers.add_parser("finetune_completions.create")
    #
    # req = sub.add_argument_group("required arguments")
    # opt = sub.add_argument_group("optional arguments")
    #
    # req.add_argument(
    #     "-m",
    #     "--model_id",
    #     required=True,
    #     help="Model id."
    # )
    #
    # req.add_argument(
    #     "-t",
    #     "--text",
    #     required=True,
    #     help="Input messages."
    # )
    #
    # opt.add_argument(
    #     "--stream", help="Stream messages as they're ready.", action="store_true"
    # )
    #
    # for param in finetune_completions_option_params():
    #     opt.add_argument(f"--{param[0]}", help=param[1])
    #
    # sub.set_defaults(func=FinetuneCompletion.create)



import json
from urllib.parse import quote_plus
from typing import cast
import sensenova
from sensenova import api_requestor, util

from sensenova.api_resources.abstract import DeletableAPIResource, ListableAPIResource, CreatableAPIResource, FileableAPIResource, DownloadableAPIResource, UploadableAPIResource


class KnowledgeBase(ListableAPIResource, DeletableAPIResource, CreatableAPIResource, DownloadableAPIResource, UploadableAPIResource, FileableAPIResource):
    OBJECT_NAME = "llm.knowledge-bases"



import asyncio
import json
import platform
import warnings
from contextlib import asynccontextmanager
from typing import Tuple, Union, Iterator, Optional, Dict, AsyncGenerator, AsyncIterator
from urllib.parse import urlencode, urlsplit, urlunsplit
import threading

import aiohttp
import requests
from json import JSONDecodeError

import sensenova
from sensenova import util, error, version
from sensenova.sensenova_response import SensenovaResponse

MAX_CONNECTION_RETRIES = 2
_thread_context = threading.local()


def _build_api_url(url, query):
    scheme, netloc, path, base_query, fragment = urlsplit(url)

    if base_query:
        query = "%s%s" % (base_query, query)

    return urlunsplit((scheme, netloc, path, query, fragment))


def _make_session() -> requests.Session:
    if sensenova.requestssession:
        if isinstance(sensenova.requestssession, requests.Session):
            return sensenova.requestssession
        return sensenova.requestssession()
    if not sensenova.verify_ssl_certs:
        warnings.warn("verify_ssl_certs is ignored; sensenova always verifies.")
    s = requests.Session()
    s.mount(
        "https://",
        requests.adapters.HTTPAdapter(max_retries=MAX_CONNECTION_RETRIES)
    )
    return s


def parse_stream_helper(line: str) -> Optional[str]:
    # TODO: Adapt to chat
    if line:
        if line.strip() == b"data:[DONE]":
            return None

        if line.startswith(b"data:"):
            line = line[len(b"data:"):]
            return line.decode("utf-8")
        else:
            return None
    return None


def parse_stream(rbody: Iterator[bytes]) -> Iterator[str]:
    for line in rbody:
        _line = parse_stream_helper(line)
        if _line is not None:
            yield _line


async def parse_stream_async(rbody: aiohttp.StreamReader) -> str:
    async for line in rbody:
        _line = parse_stream_helper(line)
        if _line is not None:
            yield _line


class APIRequestor:
    def __init__(self, access_key_id=None,
                 secret_access_key=None, api_base=None, api_version=None):
        self.access_key_id = access_key_id or util.default_access_key_id()
        self.secret_access_key = secret_access_key or util.default_secret_access_key()
        self.api_base = api_base or sensenova.api_base

    @classmethod
    def format_app_info(cls, info):
        s = info["name"]
        if info["version"]:
            s += "/%s" % (info["version"],)
        if info["url"]:
            s += " (%s)" % (info["url"],)
        return s

    def request(self, method: str, url: str, params=None, headers=None, files=None, stream: bool = False) -> Tuple[
        Union[SensenovaResponse, Iterator[SensenovaResponse]], bool, str, str]:
        result = self.request_raw(
            method.lower(),
            url,
            params=params,
            supplied_headers=headers,
            files=files,
            stream=stream,
        )
        resp, got_stream = self._interpret_response(result, stream)
        return resp, got_stream, self.access_key_id, self.secret_access_key

    async def arequest(
        self,
        method: str,
        url: str,
        params=None,
        headers=None,
        files=None,
        stream: bool = False
    ) -> Tuple[Union[SensenovaResponse, AsyncGenerator[SensenovaResponse, None]], bool, str, str]:
        ctx = aiohttp_session()
        session = await ctx.__aenter__()

        try:
            result = await self.arequest_raw(
                method.lower(),
                url,
                session,
                params=params,
                supplied_headers=headers,
                files=files,
            )
            resp, got_stream = await self._interpret_async_response(result, stream)
        except Exception:
            await ctx.__aexit__(None, None, None)
            raise
        if got_stream:
            async def wrap_resp():
                assert isinstance(resp, AsyncGenerator)
                try:
                    async for r in resp:
                        yield r
                finally:
                    await ctx.__aexit__(None, None, None)

            return wrap_resp(), got_stream, self.access_key_id, self.secret_access_key
        else:
            await ctx.__aexit__(None, None, None)
            return resp, got_stream, self.access_key_id, self.secret_access_key

    def _interpret_response_line(
        self, rbody: str, rcode: int, rheaders, stream: bool
    ) -> SensenovaResponse:
        if rcode == 204:
            return SensenovaResponse(None, rheaders)

        if rcode == 503:
            raise error.ServiceUnavailableError(
                "The server is overloaded or not ready yet.",
                rbody,
                rcode,
                headers=rheaders,
            )
        try:
            if 'text/plain' in rheaders.get('Content-Type', ''):
                data = rbody
            else:
                data = json.loads(rbody)
        except (JSONDecodeError, UnicodeDecodeError) as e:
            try:
                if self.api_base == 'https://aoss' and rbody == '':
                    data = {
                        "info": "Created"
                    }
                else:
                    data = eval(rbody)

            except Exception as exp:
                raise error.APIError(
                    f"HTTP code {rcode} from API ({rbody})", rbody, rcode, headers=rheaders
                ) from e
        resp = SensenovaResponse(data, rheaders)
        stream_error = stream and "error" in resp.data
        if stream_error or not 200 <= rcode < 300:
            raise self.handle_error_response(
                rbody, rcode, resp.data, rheaders, stream_error=stream_error
            )
        return resp

    def _interpret_response(self, result: requests.Response, stream: bool) -> Tuple[
        Union[SensenovaResponse, Iterator[SensenovaResponse]], bool]:
        if stream and "text/event-stream" in result.headers.get("Content-Type", ""):
            return (
                self._interpret_response_line(
                    line, result.status_code, result.headers, stream=True
                )
                for line in parse_stream(result.iter_lines())
            ), True
        else:
            return (
                self._interpret_response_line(
                    result.content.decode('utf-8'),
                    result.status_code,
                    result.headers,
                    stream=False
                ),
                False
            )

    def request_raw(self, method, url, *, params=None, supplied_headers: Optional[Dict[str, str]] = None, files=None,
                    stream: bool = False) -> requests.Response:
        abs_url, headers, data = self._prepare_request_raw(
            url, supplied_headers, method, params, files
        )

        if not hasattr(_thread_context, "session"):
            _thread_context.session = _make_session()
        try:
            result = _thread_context.session.request(
                method,
                abs_url,
                headers=headers,
                data=data,
                files=files,
                stream=stream,
                verify=sensenova.verify_ssl_certs
            )
        except requests.exceptions.RequestException as e:
            raise error.APIConnectionError(
                "Error communicating with Sensenova: {}".format(e)
            ) from e
        util.log_debug(
            "Sensenova API response",
            path=abs_url,
            response_code=result.status_code,
        )

        if sensenova.log == 'debug':
            util.log_debug(
                "API response body", body=result.content, headers=result.headers
            )
        return result

    async def arequest_raw(
        self,
        method,
        url,
        session,
        *,
        params=None,
        supplied_headers: Optional[Dict[str, str]] = None,
        files=None,
    ) -> aiohttp.ClientResponse:
        abs_url, headers, data = self._prepare_request_raw(
            url, supplied_headers, method, params, files
        )

        if files:
            data, content_type = requests.models.RequestEncodingMixin._encode_files(
                files, data
            )
            headers["Content-Type"] = content_type

        request_kwargs = {
            "method": method,
            "url": abs_url,
            "headers": headers,
            "data": data,
        }

        try:
            result = await session.request(**request_kwargs)
            util.log_info(
                "Sensenova API response",
                path=abs_url,
                response_code=result.status,
            )
            return result
        except aiohttp.ClientError as e:
            raise error.APIConnectionError("Error communicating with Sensenova") from e

    def _prepare_request_raw(self, url, supplied_headers, method, params, files) -> Tuple[
        str, Dict[str, str], Optional[bytes]]:
        abs_url = "%s%s" % (self.api_base, url)
        headers = self._validate_headers(supplied_headers)

        data = None
        if method == "get" or method == "delete":
            if params:
                encoded_params = urlencode(
                    [(k, v) for k, v in params.items() if v is not None]
                )
                abs_url = _build_api_url(abs_url, encoded_params)
        elif method in {"post", "put"}:
            if self.api_base == 'https://aoss':
                data = params
                headers = {"Content-Type": "application/octet-stream"}
            else:
                if params and files:
                    data = params
                if params and not files:
                    data = json.dumps(params).encode()
                    headers["Content-Type"] = "application/json"
        else:
            raise error.APIConnectionError(
                "Unrecognized HTTP method %r. This may indicate a bug in the "
                "Sensenova bindings. Please contact us" % method
            )

        headers = self.request_headers(method, headers)

        util.log_debug("Request to Sensenova API", method=method, path=abs_url)
        util.log_debug("Post details", data=data)

        return abs_url, headers, data

    def _validate_headers(self, supplied_headers: Optional[Dict[str, str]]) -> Dict[str, str]:
        headers: Dict[str, str] = {}
        if supplied_headers is None:
            return headers

        if not isinstance(supplied_headers, dict):
            raise TypeError("Headers must be a dictionary")

        for k, v in supplied_headers.items():
            if not isinstance(k, str):
                raise TypeError("Header keys must be strings")
            if not isinstance(v, str):
                raise TypeError("Header values must be strings")
            headers[k] = v

        return headers

    def request_headers(self, method: str, extra) -> Dict[str, str]:
        user_agent = "Sensenova/v1 PythonBindings/%s" % (version.VERSION,)
        if sensenova.app_info:
            user_agent += " " + self.format_app_info(sensenova.app_info)

        uname_without_node = " ".join(
            v for k, v in platform.uname()._asdict().items() if k != "node"
        )

        ua = {
            "bindings_version": version.VERSION,
            "httplib": "requests",
            "lang": "python",
            "lang_version": platform.python_version(),
            "platform": platform.platform(),
            "publisher": "sensenova",
            "uname": uname_without_node,
        }

        if sensenova.app_info:
            ua['application'] = sensenova.app_info

        if self.api_base != 'https://aoss':
            headers = {
                "X-Sensenova-Client-User-Agent": json.dumps(ua),
                "User-Agent": user_agent,
            }
            headers.update(util.ak_sk_to_header(self.access_key_id, self.secret_access_key))
        else:
            headers = {}
        if sensenova.debug:
            headers["Sensenova-Debug"] = "true"
        headers.update(extra)

        return headers

    @classmethod
    def handle_error_response(cls, rbody, rcode, resp, rheaders, stream_error=False):
        try:
            error_data = resp["error"]
        except (KeyError, TypeError):
            raise error.APIError(
                "Invalid response object from API: %r (HTTP response code "
                "was %d)" % (rbody, rcode),
                rbody,
                rcode,
                resp,
            )

        util.log_info(
            "Sensenova API error received",
            error_code=error_data.get("code"),
            error_message=error_data.get("message"),
            stream_error=stream_error,
        )

        if rcode in [400, 404, 415]:
            return error.InvalidRequestError(
                error_data.get("message"),
                None,
                error_data.get("code"),
                rbody,
                rcode,
                resp,
                rheaders,
            )
        elif rcode == 429:
            return error.RateLimitError(
                error_data.get("message"), rbody, rcode, resp, rheaders
            )
        elif rcode == 401:
            return error.AuthenticationError(
                error_data.get("message"), rbody, rcode, resp, rheaders
            )
        elif rcode == 403:
            return error.PermissionError(
                error_data.get("message"), rbody, rcode, resp, rheaders
            )
        elif rcode == 409:
            return error.TryAgain(
                error_data.get("message"), rbody, rcode, resp, rheaders
            )
        elif stream_error:
            parts = [error_data.get("message"), "(Error occurred while streaming.)"]
            message = " ".join([p for p in parts if p is not None])
            return error.APIError(message, rbody, rcode, resp, rheaders)
        else:
            return error.APIError(
                f"{error_data.get('message')} {rbody} {rcode} {resp} {rheaders}",
                rbody,
                rcode,
                resp,
                rheaders,
            )

    async def _interpret_async_response(self, result: aiohttp.ClientResponse, stream: bool) -> Tuple[Union[
        SensenovaResponse, AsyncGenerator[SensenovaResponse, None]], bool]:
        if stream and "text/event-stream" in result.headers.get("Content-Type", ""):
            return (
                self._interpret_response_line(
                    line, result.status, result.headers, stream=True
                )
                async for line in parse_stream_async(result.content)
            ), True
        else:
            try:
                await result.read()
            except aiohttp.ClientError as e:
                util.log_warn(e, body=result.content)
            return (
                self._interpret_response_line(
                    (await result.read()).decode("utf-8"),
                    result.status,
                    result.headers,
                    stream=False,
                ),
                False,
            )


@asynccontextmanager
async def aiohttp_session() -> AsyncIterator[aiohttp.ClientSession]:
    user_set_session = sensenova.aiosession.get()
    if user_set_session:
        yield user_set_session
    else:
        async with aiohttp.ClientSession() as session:
            yield session



from urllib.parse import quote_plus, urlencode

import sensenova
from sensenova import error, api_requestor, util
from sensenova.sensenova_object import SensenovaObject


class APIResource(SensenovaObject):
    api_prefix = ""

    @classmethod
    def retrieve(cls, id, access_key_id=None, secret_access_key=None, **params):
        instance = cls(id=id, access_key_id=access_key_id, secret_access_key=secret_access_key, **params)
        instance.refresh()
        return instance

    @classmethod
    def aretrieve(
        cls, id, access_key_id=None, secret_access_key=None, request_id=None, request_timeout=None, **params
    ):
        instance = cls(id=id, access_key_id=access_key_id, secret_access_key=secret_access_key, **params)
        return instance.arefresh()

    def refresh(self):
        self.refresh_from(
            self.request(
                "get",
                self.instance_url()
            )
        )
        return self

    async def arefresh(self):
        self.refresh_from(
            await self.arequest(
                "get",
                self.instance_url(),
            )
        )
        return self

    @classmethod
    def class_url(cls):
        if cls == APIResource:
            raise NotImplementedError(
                "APIResource is an abstract class. You should perform actions on its subclasses."
            )

        base = cls.OBJECT_NAME.replace(".", "/")
        if cls.api_prefix:
            return "/%s/%s" % (cls.api_prefix, base)
        return "/%s" % base

    def instance_url(self, operation=None):
        id = self.get("id")

        if not isinstance(id, str):
            raise error.InvalidRequestError(
                "Could not determine which URL to request: %s instance "
                "has invalid ID: %r, %s. ID should be of type `str` (or"
                " `unicode`)" % (type(self).__name__, id, type(id)),
                "id",
            )

        extn = quote_plus(id)
        base = self.class_url()
        return "%s/%s" % (base, extn)

    @classmethod
    def _static_request(
        cls,
        method_,
        url_,
        access_key_id=None, secret_access_key=None,
        api_base=None,
        **params
    ):
        requestor = api_requestor.APIRequestor(
            access_key_id, secret_access_key,
            api_base=api_base,
        )
        response, _, access_key_id, secret_access_key = requestor.request(
            method_, url_, params
        )
        return util.convert_to_sensenova_object(response, access_key_id, secret_access_key, )

    @classmethod
    async def _astatic_request(
        cls,
        method_,
        url_,
        access_key_id=None, secret_access_key=None,
        api_base=None,
        **params
    ):
        requestor = api_requestor.APIRequestor(
            access_key_id, secret_access_key,
            api_base=api_base,
        )
        response, _, access_key_id, secret_access_key = await requestor.arequest(
            method_, url_, params
        )
        return response



import json
from typing import cast
from urllib.parse import quote_plus

import sensenova
from sensenova import api_requestor, util
from sensenova.api_resources.abstract import DeletableAPIResource, ListableAPIResource, CreatableAPIResource, FileableAPIResource, DownloadableAPIResource, UploadableAPIResource


class Dataset(ListableAPIResource, DeletableAPIResource, CreatableAPIResource, FileableAPIResource, DownloadableAPIResource, UploadableAPIResource):
    OBJECT_NAME = "llm.fine-tune.datasets"



from sensenova import error
from sensenova.api_resources.abstract import EngineAPIResource


class ChatCompletion(EngineAPIResource):
    OBJECT_NAME = "llm.chat-completions"

    @classmethod
    def create(cls, *args, **kwargs):
        messages = kwargs.get("messages", None)

        if not messages:
            raise error.InvalidRequestError(
                "The parameter messages is required"
            )
        return super().create(*args, **kwargs)

    @classmethod
    async def acreate(cls, *args, **kwargs):
        messages = kwargs.get("messages", None)

        if not messages:
            raise error.InvalidRequestError(
                "The parameter messages is required"
            )

        return await super().acreate(*args, **kwargs)



import json
import subprocess
from tempfile import NamedTemporaryFile


def test_prepare_data() -> None:
    unprepared_training_data = [
        {
            "instruction": "我们如何在日常生活中减少用水？",
            "input": "",
            "output": "1. 使用节水装置，如节水淋浴喷头和水龙头。 \n2. 使用水箱或水桶收集家庭废水，例如洗碗和洗浴。 \n3. 在社区中提高节水意识。 \n4. 检查水管和灌溉系统的漏水情况，并及时修复它们。 \n5. 洗澡时间缩短，使用低流量淋浴头节约用水。 \n6. 收集雨水，用于园艺或其他非饮用目的。 \n7. 刷牙或擦手时关掉水龙头。 \n8. 减少浇水草坪的时间。 \n9. 尽可能多地重复使用灰水（来自洗衣机、浴室水槽和淋浴的水）。 \n10. 只购买能源效率高的洗碗机和洗衣机。"
        },
        {
            "instruction": "编辑文章，使其更吸引读者。",
            "input": "自主机器人是计算机控制的机器，被编程执行特定任务而不需要任何人类输入。自主机器人在各个行业中被越来越广泛地应用，从制造业到医疗保健再到安全。",
            "output": "自主机器人是计算机控制的机器，被编程执行特定任务而不需要任何人类输入，从而实现了新的效率、精确度和可靠性水平。自主机器人在各个行业中被越来越广泛地应用，从制造业，它们可以使用精度和一致的质量组装复杂的组件，到医疗保健，可以协助进行医疗测试和处理，再到安全，可以监控大面积地区，保障人们和财产的安全。自主机器人还可以减少在危险或有害环境中的错误和增加安全，在工业流程的检查或维修期间等。由于其多样性，自主机器人将彻底改变我们工作方式的方式，使任务变得更加简单、快速，最终更加愉悦。"
        },
        {
            "instruction": "政府可以采取哪些策略来减少空气污染？",
            "input": "",
            "output": "1. 实施强制的车辆排放标准和基于激励的计划，以降低车辆的碳足迹。\n2. 增加公共交通工具，减少公众对车辆的依赖。\n3. 增加对空气污染的影响的认识，鼓励市民减少污染物的生成。\n4. 投资于可再生能源的研究和开发，如太阳能和风能。\n5. 在工厂和发电厂安装空气污染控制装置，例如洗涤器。\n6. 对车辆和工厂使用清洁燃料。\n7. 实施更好的城市规划和控制拓展。\n8. 改善农业效率，减少化肥和杀虫剂的使用。\n9. 种植更多的树木以减少空气污染。\n10. 减少木材、煤炭和生物质的燃烧。"
        },
        {
            "instruction": "我们如何在日常生活中减少用水？",
            "input": "",
            "output": "1. 使用节水装置，如节水淋浴喷头和水龙头。 \n2. 使用水箱或水桶收集家庭废水，例如洗碗和洗浴。 \n3. 在社区中提高节水意识。 \n4. 检查水管和灌溉系统的漏水情况，并及时修复它们。 \n5. 洗澡时间缩短，使用低流量淋浴头节约用水。 \n6. 收集雨水，用于园艺或其他非饮用目的。 \n7. 刷牙或擦手时关掉水龙头。 \n8. 减少浇水草坪的时间。 \n9. 尽可能多地重复使用灰水（来自洗衣机、浴室水槽和淋浴的水）。 \n10. 只购买能源效率高的洗碗机和洗衣机。"
        }
    ]

    with NamedTemporaryFile(suffix=".json", mode="w") as training_data:
        training_data.write(json.dumps(unprepared_training_data, ensure_ascii=False))
        training_data.flush()

        prepared_data_cmd_output = subprocess.run(
            [f"sensenova tools fine_tunes.prepare_data -f {training_data.name}"],
            # ['ls'],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            encoding="utf-8",
            shell=True,
        )

        return prepared_data_cmd_output.stdout


if __name__ == '__main__':
    test_prepare_data()



import sys

import sensenova

# result = sensenova.ChatCompletion.create(
#     messages=[{"role": "user", "content": "Say this is a test!"}]
# )
stream = False
resp = sensenova.ChatCompletion.create(
    messages=[{"role": "user", "content": "我们如何在日常生活中减少用水"}],
    model="llama-7b-test",
    stream=stream,
)

if not stream:
    resp = [resp]
for part in resp:
    choices = part['data']["choices"]
    for c_idx, c in enumerate(choices):
        if len(choices) > 1:
            sys.stdout.write("===== Chat Completion {} =====\n".format(c_idx))
        if stream:
            delta = c.get("delta")
            if delta:
                sys.stdout.write(delta)
        else:
            sys.stdout.write(c["message"])
            if len(choices) > 1:  # not in streams
                sys.stdout.write("\n")
        sys.stdout.flush()



from sensenova import api_resources


OBJECT_CLASSES = {
    "dataset": api_resources.Dataset,
    "model": api_resources.Model,
    "fine-tune": api_resources.FineTune,
    "serving": api_resources.Serving
}



from sensenova.api_resources.error_object import ErrorObject
from sensenova.api_resources.dataset import Dataset
from sensenova.api_resources.model import Model
from sensenova.api_resources.fine_tune import FineTune
from sensenova.api_resources.serving import Serving
from sensenova.api_resources.chat_completion import ChatCompletion
from sensenova.api_resources.finetune_completion import FinetuneCompletion
from sensenova.api_resources.knowledge_base import KnowledgeBase



import os
import sys
from contextvars import ContextVar
from typing import Optional, Callable, Union, TYPE_CHECKING

import requests

from sensenova.version import VERSION

from sensenova.api_resources import (
    Dataset,
    Model,
    FineTune,
    Serving,
    ChatCompletion,
    FinetuneCompletion,
    KnowledgeBase
)
from sensenova.error import APIError, InvalidRequestError, SensenovaError

if TYPE_CHECKING:
    import requests
    from aiohttp import ClientSession

verify_ssl_certs = True
access_key_id = os.environ.get("SENSENOVA_ACCESS_KEY_ID")
secret_access_key = os.environ.get("SENSENOVA_SECRET_ACCESS_KEY")
api_base = os.environ.get("SENSENOVA_API_BASE", "https://api.sensenova.cn/v1")
app_info = None
debug = False
log = None

requestssession: Optional[
    Union["requests.Session", Callable[[], "requests.Session"]]
] = None

aiosession: ContextVar[Optional["ClientSession"]] = ContextVar(
    "aiohttp-session", default=None
)

__version__ = VERSION
__all__ = [
    "APIError",
    "InvalidRequestError",
    "SensenovaError",
    "Dataset",
    "Model",
    "FineTune",
    "Serving",
    "ChatCompletion",
    "FinetuneCompletion",
    "KnowledgeBase"
]
